from dataclasses import dataclass
from pathlib import Path

import yaml

from src.exceptions import DefinitionError


@dataclass
class Snippet:
    markdown: str
    sulg: dict


def read_markdown(file: Path, depth: int = 0) -> Snippet:
    """
    Reads in a markdown file and applies pre-processing.

    args:
        file: The Markdown file to read.

        depth: The number of headers deep.
            For example, a raw '## header' with a depth of
            3 will be changed to '##### header'
    """
    with open(file) as fp:
        raw_markdown = fp.read().splitlines()

    # Parse Slug
    slug = {}
    if "---" == raw_markdown[0]:
        raw_markdown = raw_markdown[1:]
        end_slug = raw_markdown.index("---")
        raw_slug, raw_markdown = raw_markdown[:end_slug], raw_markdown[end_slug + 1 :]
        slug = yaml.load("\n".join(raw_slug), Loader=yaml.Loader)

    # TODO: Implement depth
    for i, line in enumerate(raw_markdown):
        if line.strip().startswith("#"):
            raw_markdown[i] = "#" * depth + line.strip()

    return Snippet("\n".join(raw_markdown), slug)


def parse_yaml(file: Path) -> dict:
    with open(file) as fp:
        return yaml.load(fp, Loader=yaml.Loader)


def build_document(definition: Path) -> str:
    structure = parse_yaml(definition)
    document = ""
    base_path = definition.parent.parent / "content"
    document += build_header(structure.get("header", {}))
    document += "\n\n"
    document += build_body(structure["resume"], base_path)
    return document


def build_header(structure: dict) -> str:
    if not structure:
        return ""
    return "{name}\n\n{email}\n\ngithub.com/{github}\n\n{website}".format(**structure)


def build_body(structure: dict, base_path: Path):
    document = ""
    for category, snippets in structure.items():
        document += f"\n\n# {category.strip().capitalize()}\n\n"
        for snippet in snippets:
            match snippet:
                case "---":
                    document += snippet
                case _:
                    path = base_path / category / snippet
                    if not path.exists():
                        raise DefinitionError(
                            f"{path=} Does not exist, cannot create document"
                        )
                    document += read_markdown(
                        base_path / category / snippet, depth=1
                    ).markdown
            document += "\n\n"
    return document



from pathlib import Path

import typer

from src.builder import build_document

app = typer.Typer()


@app.command()
def build(path: Path, preview: bool = False):
    doc = build_document(path)
    if preview:
        from rich import print as rprint
        from rich.markdown import Markdown

        rprint(Markdown(doc))
        return
    print(doc)


if __name__ == "__main__":
    app()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['rattlesume']

package_data = \
{'': ['*']}

install_requires = \
['pyyaml>=6.0,<7.0', 'typer[all]>=0.7.0,<0.8.0']

setup_kwargs = {
    'name': 'rattlesume',
    'version': '0.1.0',
    'description': '',
    'long_description': '# Rattlesume\n\nA tool for building resumes by combining markdown snippets into a single\ndocument using `yaml` definitions, then transformed into a PDF `pandoc`. Allows\ndevelopers to store their resumes in git, and present them in a common format.\nDifferent yaml definitions can be used to craft bespoke resumes from a collection\nof markdown snippets.\n\n# Usage\n\n## Snippets\n\nTo start, create a `content` directory with a subdirectory\nto organize a class of snippets. **The name of the\nsubdirectory will be the same as the header in the final resume.**\n\n```\n\uf115 example/content/\n├── \uf115 experience/\n│  └── \uf48a FooBar.md\n└── \uf115 projects/\n   └── \uf48a Rattlusume.md\n```\n\nEach of these markdown files are called \'snippets\'. They are\ntypically a short summary & include relevant information such as\ndates, job titles, etc.\n\n```markdown\n---\nfilename: FooBar.md\n---\n# FooBar Inc.\n## Sr. Foobar Enigneer, March 5th, 2015 - Present\n\nResponsible for announcing the zero-modulate of the counting numbers greater than\nzero with respect to three and five. Saved FooBar Inc. $15 annually by implementing\nan `O(n)` solution.\n```\n\n> Snippets can contain an optional slug at the very top, denoted by `---`.\n> This slug must be valid yaml. Currently, it is parsed, but not used.\n\n\n## Definitions\n\nEach unique resume has a corresponding definition file.\nThere are currently two required top-level values. `header`,\nand `resume`.\n\n### `header`\n\nThe format of this value is still in flux.\n\n### `resume`\n\nThe `resume` section has arbitrary sub-key corresponding to the\nnames of the subdirectories the snippets are located in.\n\nEach of these sub-keys take a list of values, which are ether file names\nor special values.\n\nFor example, the aforementioned \'FooBar.md\' could be defined as follows:\n\n```yaml\nheader:\n  ...\n\nresume:\n  experience:\n    - "---"\n    - "FooBar.md"\n```\n\nThere is currently only one special value,\n`"---"`, which denotes a horizontal rule.\n',
    'author': 'Kelton Bassingthwaite',
    'author_email': 'keltonbassingthwaite@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)
